/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.xerces;

import java.io.IOException;
import java.io.OutputStream;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.DocumentFragment;

/**
 * CustomXmlSerializer extends the default XmlSerializer to correct a small bug
 * in it This class should then be removed as soon as this bug is corrected. The
 * bug is that serializing a document fragment does not call serializePreRoot,
 * while it should be the case. Method serialize has been rewritten to correct
 * that
 * 
 * @author Sebastien Ponce
 */
public class CustomXMLSerializer extends XMLSerializer {

    /**
     * Constructs a new serializer that writes to the specified output stream
     * using the specified output format. If <tt>format</tt> is null, will use
     * a default output format.
     * 
     * @param output The output stream to use
     * @param format The output format to use, null for the default
     */
    public CustomXMLSerializer(OutputStream output, OutputFormat format) {
        super(output, format);
    }

    /**
     * Serializes the DOM document fragmnt using the previously specified writer
     * and output format. Throws an exception only if an I/O exception occured
     * while serializing.
     * 
     * @param frag The element to serialize
     * @throws IOException An I/O exception occured while serializing
     */
    public void serialize(DocumentFragment frag) throws IOException {
        prepare();
        serializeNode(frag);
        // This line is the only difference between the default implementation
        // and
        // this one.
        serializePreRoot();
        _printer.flush();
        if (_printer.getException() != null) throw _printer.getException();
    }

}