/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.cese;

import java.awt.Component;
import java.beans.BeanInfo;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ListCellRenderer;

class ModelBoxRenderer extends JLabel implements ListCellRenderer {

    /**
     * True to use full mode, false to use brief mode
     */
    private boolean mode = false;

    /**
     * Default constructor
     * 
     * @param m mode (full or brief)
     */
    public ModelBoxRenderer(boolean m) {
        mode = m;
        setOpaque(true);
        setHorizontalAlignment(LEFT);
        setVerticalAlignment(CENTER);
    }

    /**
     * Returns rendering component
     * 
     * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
     *      java.lang.Object, int, boolean, boolean)
     */
    public Component getListCellRendererComponent(JList list, Object value,
            int index, boolean isSelected, boolean cellHasFocus) {
        if (isSelected) {
            setBackground(list.getSelectionBackground());
            setForeground(list.getSelectionForeground());
        } else {
            setBackground(list.getBackground());
            setForeground(list.getForeground());
        }

        ModelInfo info = (ModelInfo) value;
        if (info != null) {
            if (mode) {
                setText("<html>" + info.getName() + "<br>"
                        + ResourceManager.getResource("Licensed_to") + " "
                        + info.getModel().getUserName() + ", "
                        + info.getModel().getOrganisation() + "<br>"
                        + ResourceManager.getResource("Serial_number") + " "
                        + info.getModel().getSerialNumber() + "<br></html>");
            } else {
                setText(info.getName());
            }
            setIcon(new ImageIcon(info.getIcon(BeanInfo.ICON_COLOR_16x16)));
            setToolTipText("<html>" + info.getShortDescription() + "</html>");
        }
        return this;
    }
}