/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese.updates;

import java.net.URL;
import java.util.Vector;
import java.util.prefs.Preferences;

import com.simulogic.cese.ResourceManager;
import com.simulogic.cese.UserHome;
import com.sun.syndication.feed.synd.SyndEntryImpl;
import com.sun.syndication.feed.synd.SyndFeed;
import com.sun.syndication.io.SyndFeedInput;
import com.sun.syndication.io.XmlReader;

/**
 * Fetches update information and changelogs
 */
public class UpdateFetcher {

	private static final String newsFeed = ResourceManager
			.getResource("news_feed");

	private static final String changelogFeed = ResourceManager
			.getResource("changelog_feed");

	private String title;

	private String link;

	private String pubdate;

	private Preferences preferences = null;

	private double updateInterval = 1.2096e9;

	/**
	 * Fetch update information
	 */
	public void fetchUpdate() {
		try {
			URL feedUrl = new URL(newsFeed);

			SyndFeedInput input = new SyndFeedInput();
			SyndFeed feed = input.build(new XmlReader(feedUrl));

			Vector entries = new Vector(feed.getEntries());

			SyndEntryImpl entry = (SyndEntryImpl) entries.get(0);

			setTitle(entry.getTitle());
			setLink(entry.getLink());
			setPubdate(entry.getPublishedDate().toString());
		} catch (Exception e) {
		}
	}

	/**
	 * Fetch changelog information
	 * 
	 * @return changelog string
	 */
	public String fetchChangelog() {
		String changelog = "<html>";

		try {
			URL feedUrl = new URL(changelogFeed);

			SyndFeedInput input = new SyndFeedInput();
			SyndFeed feed = input.build(new XmlReader(feedUrl));

			Vector entries = new Vector(feed.getEntries());
			for (int i = 0; i < entries.size(); i++) {
				SyndEntryImpl entry = (SyndEntryImpl) entries.get(i);

				changelog += "<p>" + entry.getDescription().getValue() + "</p>";
			}
		} catch (Exception e) {
		}

		changelog += "</html>";
		return changelog;
	}

	/**
	 * Returns true if new version is available
	 * 
	 * @return true if new version is available
	 */
	public boolean isnewVersionAvailable() {
		fetchUpdate();
		double current = 0.0;
		double fetched = 0.0;

		int dot = getTitle().indexOf('.');
		String subver = getTitle().substring(dot + 1,
				getTitle().indexOf(' ', dot));

		fetched = new Double(subver).doubleValue();

		dot = UserHome.CESE_VERSION_MINOR.indexOf('.');
		subver = UserHome.CESE_VERSION_MINOR.substring(dot + 1,
				UserHome.CESE_VERSION_MINOR.length());

		current = new Double(subver).doubleValue();

		if (fetched > current)
			return true;

		return false;
	}

	/**
	 * Saves current time for updates
	 */
	public void setUpdateTime() {
		preferences = Preferences.userNodeForPackage(this.getClass());
		preferences.putLong("update", System.currentTimeMillis());
	}

	/**
	 * Returns true if it is time to check for updates
	 * 
	 * @return true if it is time to check for updates
	 */
	public boolean isUpdateTime() {
		preferences = Preferences.userNodeForPackage(this.getClass());

		long last = preferences.getLong("update", 0);
		long current = System.currentTimeMillis();
		
		if (last == 0) {
			setUpdateTime();
			return false;
		}

		if ((current-last) > updateInterval)
			return true;

		return false;
	}

	/**
	 * Sets
	 * 
	 * @param title
	 *            the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * Returns
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Sets
	 * 
	 * @param link
	 *            the link to set
	 */
	public void setLink(String link) {
		this.link = link;
	}

	/**
	 * Returns
	 * 
	 * @return the link
	 */
	public String getLink() {
		return link;
	}

	/**
	 * Sets
	 * 
	 * @param pubdate
	 *            the pubdate to set
	 */
	public void setPubdate(String pubdate) {
		this.pubdate = pubdate;
	}

	/**
	 * Returns
	 * 
	 * @return the pubdate
	 */
	public String getPubdate() {
		return pubdate;
	}
}
