/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 * 
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this library; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 *  
 */

package com.simulogic.dataview.actions;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import com.simulogic.cese.UserHome;
import com.simulogic.dataview.ResourceManager;
import com.simulogic.dataview.ViewContent;
import com.simulogic.dataview.export.AsciiExport;
import com.simulogic.dataview.export.AtfExport;
import com.simulogic.dataview.export.NetcdfExport;
import com.simulogic.dataview.export.PngExport;
import com.simulogic.dataview.export.SvgExport;
import com.simulogic.utils.ExceptionDialog;
import com.simulogic.utils.ExtensionFileFilter;

/**
 * ExportRes action implementation
 */
public class ExportResAction extends com.simulogic.utils.ActionSupport implements
		java.beans.PropertyChangeListener {

	private java.io.File dir;

	private java.io.File file;

	private JFileChooser filechooser;

	private com.simulogic.dataview.DataFrame ifr = null;

	private com.simulogic.dataview.DataDesktop desktop;

	/**
	 * Creates new action
	 * 
	 * @param d
	 *            desktop
	 */
	public ExportResAction(com.simulogic.dataview.DataDesktop d) {
		this();
		desktop = d;
		desktop.addPropertyChangeListener(this);
	}

	/**
	 * Creates new action
	 */
	public ExportResAction() {
		super(ResourceManager.getResource("Export_Results..."), ResourceManager
				.getResource("Export_results"), ResourceManager
				.getResource("export_image"));
	}

	/**
	 * Action interface implementation
	 * 
	 * @param evt
	 *            event
	 */
	public void actionPerformed(java.awt.event.ActionEvent evt) {
		if (dir == null)
			dir = new File(UserHome.CESE_HOME + UserHome.FS
					+ ResourceManager.getResource("res_dir"));

		filechooser = new JFileChooser(dir);
		filechooser.setDialogTitle(ResourceManager
				.getResource("Export_Results"));

		filechooser.resetChoosableFileFilters();
		filechooser.setAcceptAllFileFilterUsed(true);

		com.simulogic.utils.ExtensionFileFilter asciiFilter = new com.simulogic.utils.ExtensionFileFilter(
				ResourceManager.getResource("ascii_extension"), ResourceManager
						.getResource("ASCII_Files"));
		filechooser.addChoosableFileFilter(asciiFilter);

		com.simulogic.utils.ExtensionFileFilter atfFilter = new com.simulogic.utils.ExtensionFileFilter(
				ResourceManager.getResource("atf_extension"), ResourceManager
						.getResource("ATF_Files"));
		filechooser.addChoosableFileFilter(atfFilter);

		com.simulogic.utils.ExtensionFileFilter netcdfFilter = new com.simulogic.utils.ExtensionFileFilter(
				ResourceManager.getResource("nc_extension"), ResourceManager
						.getResource("NetCDF_Files"));
		filechooser.addChoosableFileFilter(netcdfFilter);

		com.simulogic.utils.ExtensionFileFilter pngFilter = new com.simulogic.utils.ExtensionFileFilter(
				ResourceManager.getResource("png_extension"), ResourceManager
						.getResource("PNG_Files"));
		filechooser.addChoosableFileFilter(pngFilter);
		
		com.simulogic.utils.ExtensionFileFilter svgFilter = new com.simulogic.utils.ExtensionFileFilter(
				ResourceManager.getResource("svg_extension"), ResourceManager
						.getResource("SVG_Files"));
		filechooser.addChoosableFileFilter(svgFilter);

		filechooser.setFileFilter(asciiFilter);

		int retVal = filechooser.showSaveDialog(desktop);
		if (retVal == JFileChooser.APPROVE_OPTION) {
			file = filechooser.getSelectedFile();
			dir = filechooser.getCurrentDirectory();
			if (file.getPath() == null)
				return;

			ExtensionFileFilter filter = (ExtensionFileFilter) filechooser
					.getFileFilter();

			// Add default extension if no extension is given by the user
			if (filter.getExtension(file) == null)
				file = new java.io.File(file.getAbsolutePath() + "."
						+ filter.getExtension());

			if (file.exists()) {
				int n = JOptionPane.showConfirmDialog(desktop, ResourceManager
						.getResource("Overwrite_file_")
						+ file.getPath() + " ?", ResourceManager
						.getResource("File_already_exists"),
						JOptionPane.YES_NO_OPTION);
				if (n == JOptionPane.NO_OPTION)
					return;
			}

			ifr = desktop.getActiveFrame();
			if (ifr != null) {
				com.simulogic.utils.SwingWorker worker = new com.simulogic.utils.SwingWorker() {
					Exception ex = null;

					public Object construct() {
						try {
							ex = null;

							ViewContent content = ifr.getSelectedContent();

							ExtensionFileFilter filter = (ExtensionFileFilter) filechooser
									.getFileFilter();
							if (filter.getExtension(file).equals(
									ResourceManager
											.getResource("res_extension")))
								new AsciiExport(content).export(file.getPath());
							else if (filter.getExtension(file).equals(
									ResourceManager
											.getResource("atf_extension")))
								new AtfExport(content).export(file.getPath());
							else if (filter
									.getExtension(file)
									.equals(
											ResourceManager
													.getResource("nc_extension")))
								new NetcdfExport(content)
										.export(file.getPath());
							else if (filter.getExtension(file).equals(
									ResourceManager
											.getResource("png_extension")))
								new PngExport(content).export(file.getPath());
							else if (filter.getExtension(file).equals(
									ResourceManager
											.getResource("svg_extension")))
								new SvgExport(content).export(file.getPath());
							else {
								new AsciiExport(content).export(file.getPath());
							}
						} catch (Exception e) {
							ex = e;
						}
						return ResourceManager.getResource("Done");
					}

					public void finished() {
						desktop.getTasks().removeTask(
								ResourceManager.getResource("Export_Results")
										+ " " + file.getPath());

						if (ex != null)
							ExceptionDialog.showExceptionDialog(ResourceManager
									.getResource("Error"), ResourceManager
									.getResource("Error_saving_results_file."),
									ex);
					}
				};
				desktop.getTasks().addTask(
						ResourceManager.getResource("Export_Results") + " "
								+ file.getPath());
				worker.start();
			}
		}
	}

	/**
	 * Reflects change in the property (disables/enables) action
	 * 
	 * @param evt
	 *            event
	 */
	public void propertyChange(java.beans.PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals("frame_control"))
			this.setEnabled(((Boolean) evt.getNewValue()).booleanValue());
	}
}